<?php
declare(strict_types=1);

namespace SpaceshipDomainRegistrar;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Psr7\Request;

class SpaceshipAPI
{
    private $client;
    private $baseUri;
    private $headers;

    public function __construct(string $apiKey, string $apiSecret, string $baseUri)
    {
        $this->baseUri = rtrim($baseUri, '/');
        $this->client = new Client([
            'timeout' => 30,
            'connect_timeout' => 10,
            'http_errors' => false
        ]);

        $this->headers = [
            'Authorization' => 'Bearer ' . base64_encode("$apiKey:$apiSecret"),
            'Accept' => 'application/json',
            'Content-Type' => 'application/json',
            'User-Agent' => 'Spaceship-WHMCS/2.0'
        ];
    }

    private function request(string $method, string $endpoint, array $data = []): array
    {
        $uri = $this->baseUri . '/' . ltrim($endpoint, '/');
        $options = ['headers' => $this->headers];

        if (!empty($data)) {
            $options['json'] = $data;
        }

        try {
            $response = $this->client->request($method, $uri, $options);
            $body = $response->getBody()->getContents();
            $result = json_decode($body, true) ?? [];

            if ($response->getStatusCode() >= 400) {
                $result['error'] = true;
                $result['status'] = $response->getStatusCode();
                if (!isset($result['message'])) {
                    $result['message'] = 'API request failed';
                }
            }

            return $result;
        } catch (RequestException $e) {
            return [
                'error' => true,
                'message' => $e->getMessage(),
                'status' => $e->getCode()
            ];
        }
    }

    public function registerDomain(array $data): array
    {
        return $this->request('POST', 'domains', $data);
    }

    public function transferDomain(array $data): array
    {
        return $this->request('POST', 'domains/transfer', $data);
    }

    public function renewDomain(string $domain, int $years): array
    {
        return $this->request('POST', "domains/$domain/renew", ['years' => $years]);
    }

    public function getDomain(string $domain): array
    {
        return $this->request('GET', "domains/$domain");
    }

    public function updateNameservers(string $domain, array $nameservers): array
    {
        return $this->request('PUT', "domains/$domain/nameservers", ['nameservers' => $nameservers]);
    }

    public function setDomainLock(string $domain, bool $lock): array
    {
        return $this->request('PUT', "domains/$domain/lock", ['lock' => $lock]);
    }

    public function getAuthCode(string $domain): array
    {
        return $this->request('GET', "domains/$domain/authcode");
    }

    public function getContact(string $contactId): array
    {
        return $this->request('GET', "contacts/$contactId");
    }

    public function createContact(array $data): array
    {
        return $this->request('POST', 'contacts', $data);
    }

    public function updateContact(string $contactId, array $data): array
    {
        return $this->request('PUT', "contacts/$contactId", $data);
    }

    public function checkAvailability(string $domain): array
    {
        return $this->request('GET', "domains/check?domain=$domain");
    }

    public function getPricing(string $tld): array
    {
        return $this->request('GET', "domains/pricing/$tld");
    }

    public function getDNSSEC(string $domain): array
    {
        return $this->request('GET', "domains/$domain/dnssec");
    }

    public function updateDNSSEC(string $domain, array $dsRecords): array
    {
        return $this->request('PUT', "domains/$domain/dnssec", ['ds_records' => $dsRecords]);
    }
}